
##########################################################################
# R script to simulate the phenology of the pine processionary moth (PPM), Thaumetopoea pityocampa 
# Funded by the national grant ANR PHENEC (grant number19-CE32-0007); PhD thesis of L Poitou granted by the French region Val de Loire.
# Authors: Laura Poitou (INRAE, URZF, Orleans, France) 
#          Christelle Suppo (IRBI, University of Tours, Tours, France) 
#          Christelle Robinet (INRAE, URZF, Orleans, France)
# Date: 29 March 2021
##########################################################################


# The aim is to predict the development of each stage from egg to the last 
# larval stage L5.
# The model was calibrated on controlled condition experiment
# The developpement rate from egg to L4 stages was mesured 
# at different thermal treatments.
# Since we did not succeed in measuring development of L5,
# its development was considered as the same as L4.
# The model was validated with observed data.
# The model is built and validated for one population: Orleans population (France)

# The lines with "--------------" indicates that they should be adapted to your files and case study.



#### Set working directory ####
# Path to the folder containing the datasets (to adapt):
#--------------------------------------
setwd("C:/Users/Documents/Model_pheno")
#--------------------------------------

#### Packages to load ####

library(lubridate) # to have the yday function
library(hydroGOF) # to have the mae function
library(ggplot2) # to have ggplot function


##########################################################################
#### Input data ####
##########################################################################

# Temperature data #

# Data with 3 variables: 
# Date (format = "%Y-%mm-%dd")
# Temp (daily mean temperature in C)
# seq: a julian day sequence that does not reset to 0 at the end of the year


# to load the .csv with temperature data
#--------------------------------------
TempVF <- read.csv("Temperature.csv", header = T, sep = ";") 
#--------------------------------------
TempVF$Date <- as.Date(TempVF$Date, format = "%d/%m/%Y")

##


# Flight period #

# Date with the number of individuals caught /day or at finer time resolution (Number_adult_catches) 
# and the cumulated number (Cumulated_number_catches)
#--------------------------------------
Flight <- read.csv("Flight_Orleans_2019.csv", header = T, sep = ";" ) 
#--------------------------------------
Flight$Date <- strptime(Flight$Date, format = "%d/%m/%Y %H:%M") 
Flight$Number_adult_catches <- as.numeric(Flight$Number_adult_catches)
Flight$Cumulated_number_catches <- as.numeric(Flight$Cumulated_number_catches)


##


# Egg period #

# PPM adults have short lifespan, so to estimate egg-laying dates,
# two days were added to adults catch dates.

# BeggO is a subset of Flight with adults catch dates
Begg0 <- data.frame(subset(Flight, !Flight$Number_adult_catches == 0, select = c(Date, Number_adult_catches))) 
colnames(Begg0) <- c("Date_adult_emerg", "Male_counts")
Begg0$Yday_adult_emerg <- yday(as.character(Begg0$Date))

# The number of males and females is considered to be the same
# The pheromone trap catch only the males
# To have the number of males & females, Male_counts is multiplicated by two
Begg0$Male_Female <- Begg0$Male_counts*2


# to have the date of egg laying (adult emergence date + 2 days)
Begg0$Yday_egg_emerg <- Begg0[,3] + 2


# Date and Yday of adult and egg emergence (the origin date should be adapted)
#--------------------------------------
Begg0$Date_egg_emerg <- as.Date(Begg0[,5], origin="2018-12-31")
#--------------------------------------


##########################################################################
# Function
##########################################################################

# param : function 
# Rm: maximal development rate for each life stage (egg, L1, ... L5)
# Tm: optimal temperature for each life stage (egg, L1, ... L5)
# To: spread of the curve for each life stage (egg, L1, ... L5)

param <- function(Rm_egg = 0.02878, Tm_egg = 30.98523, To_egg = 32.31451,
                  Rm_L1 = 0.1064, Tm_L1 = 29.3348, To_L1 = 11.6322, 
                  Rm_L2 = 0.0689, Tm_L2 = 25.0753, To_L2 = 9.1466,
                  Rm_L4 = 0.05, Tm_L4 = 24, To_L4 = 10.9,
                  Rm_L5 = 0.05, Tm_L5 = 24, To_L5 = 10.9,
                  Rm_L3 = 0.04347826, Tm_L3 = 28, To_L3 = 23)
  
{


##
  
  # DevRate0: initial value of development rate (always = 0)
  # ndays: number of days during which development rate should be calculated (days with temperature data)
  DevRate0 = 0
  ndays = max(TempVF$seq)

#### Egg Development ####

tabEgg = NULL
for( i in unique(Begg0$Yday_egg_emerg)) { # loop on julian days for every egg cohort
  DevRateAccumul <- DevRate0 # accumulation of egg development rate gets back at 0 for each cohort
  j <- i # the first day of egg emergence, from which development is accumulated 
  
  while ((DevRateAccumul < 1) & (j <= ndays)) { # while loop on days
     expr <- (TempVF$seq == j)
      Templ <- TempVF$Temp[expr] # select temperature for specific day 
      DevRateTaylor <- (Rm_egg*exp((-1/2)*(((Templ - Tm_egg)/To_egg)^2))) # Taylor equation to calculate development rate
      j <- j + 1 # next day
      DevRateAccumul <- DevRateAccumul + DevRateTaylor # accumulation of development rate
  } # end while loop on days
  
  if (DevRateAccumul >= 1) { # eggs which completed their development and went to the next stage (L1)  
    tabEgg <- rbind(tabEgg, cbind(i, j, DevRateAccumul))
    colnames(tabEgg) <- c("first day", "last day", "DevRateAccumul")
  } # end if loop 1
  
  if (DevRateAccumul < 1) { # eggs which did not complete their development
    tabEgg <- rbind(tabEgg, cbind(i, NA, DevRateAccumul))
    colnames(tabEgg) <- c("first day", "last day", "DevRateAccumul")
  } # end if loop 2
  
} # end for loop on julian days

# to add the number of individuals per emergence date
pop <- tapply(Begg0$Male_counts, Begg0$Yday_egg_emerg, sum)
tabEgg <- data.frame(tabEgg, pop)

tabEggVF <- tabEgg[rep(row.names(tabEgg), tabEgg$pop), 1:4] # one row/individual


# results with minimal and maximal day for beginning 
# and end of stage
# and mean duration 
resegg <- cbind(min(tabEggVF[,1],na.rm=T), max(tabEggVF[,1],na.rm=T), 
                              min(tabEggVF[,2],na.rm=T), max(tabEggVF[,2],na.rm=T), 
                              mean(tabEggVF[,2]-tabEggVF[,1],na.rm=T))       
colnames(resegg) <-  c("min First day", "max First day", "min Last day",
                       "max Last day", "mean duration")




##


#### L1 development ####
 
DaysL1 <- tabEgg[,2][!is.na(tabEgg[,2])] # days with L1 emergence

tabL1 = NULL
for( ind in 1:length(DaysL1)) {
  i <- DaysL1[ind] # day of development
  DevRateAccumul <- DevRate0
  j <- i
  
  while((DevRateAccumul < 1) & (j <= ndays)) {
      expr <- (TempVF$seq == j) 
      Templ <- TempVF$Temp[expr]
      DevRateTaylor <- (Rm_L1*exp((-1/2)*(((Templ - Tm_L1)/To_L1)^2)))
      j <- j + 1
      DevRateAccumul <- DevRateAccumul + DevRateTaylor
    }

  if(DevRateAccumul >= 1) {
    tabL1 <- rbind(tabL1, cbind(i, j, DevRateAccumul))
    colnames(tabL1) <- c("first day", "last day", "DevRateAccumul")
  }
  
  if(DevRateAccumul < 1) {
    tabL1 <- rbind(tabL1, cbind(i, NA, DevRateAccumul))
    colnames(tabL1) <- c("first day", "last day", "DevRateAccumul")
  }
  
}


# to add the number of individuals per emergence date
popL1 <- tabEgg$pop[!is.na(tabEgg[,2])]
tabL1 <- data.frame(tabL1, popL1)

tabL1VF <- tabL1[rep(row.names(tabL1), tabL1$popL1), 1:4]


resL1 <- cbind(min(tabL1VF[,1],na.rm=T), max(tabL1VF[,1],na.rm=T), 
                              min(tabL1VF[,2],na.rm=T), max(tabL1VF[,2],na.rm=T), 
                              mean(tabL1VF[,2]-tabL1VF[,1],na.rm=T))
colnames(resL1) <-  c("min First day", "max First day", "min Last day",
                       "max Last day", "mean duration")


##


#### L2 development ####

DaysL2 <- tabL1[,2][!is.na(tabL1[,2])] # days with L2 emergence

tabL2 = NULL
for( ind in 1:length(DaysL2)) {
  i <- DaysL2[ind] # day of development
  DevRateAccumul <- DevRate0
  j <- i
  
  while((DevRateAccumul < 1) & (j <= ndays)) {
      expr <- (TempVF$seq == j) 
      Templ <- TempVF$Temp[expr]
      DevRateTaylor <- (Rm_L2*exp((-1/2)*(((Templ - Tm_L2)/To_L2)^2)))
      j <- j + 1
      DevRateAccumul <- DevRateAccumul + DevRateTaylor
    }
    
  if(DevRateAccumul >= 1) {
    tabL2 <- rbind(tabL2, cbind(i, j, DevRateAccumul))
    colnames(tabL2) <- c("first day", "last day", "DevRateAccumul")
  }
  
  if(DevRateAccumul < 1) {
    tabL2 <- rbind(tabL2, cbind(i, NA, DevRateAccumul))
    colnames(tabL2) <- c("first day", "last day", "DevRateAccumul")
  }
  
}

# to add the number of individuals per emergence date
popL2 <- tabL1$popL1[!is.na(tabL1[,2])]
tabL2 <- data.frame(tabL2, popL2)

tabL2VF <- tabL2[rep(row.names(tabL2), tabL2$popL2), 1:4]


resL2 <- cbind(min(tabL2VF[,1],na.rm=T), max(tabL2VF[,1],na.rm=T), 
                            min(tabL2VF[,2],na.rm=T), max(tabL2VF[,2],na.rm=T), 
                            mean(tabL2VF[,2]-tabL2VF[,1],na.rm=T))
colnames(resL2) <-  c("min First day", "max First day", "min Last day",
                      "max Last day", "mean duration")


##


#### L3 development ####

DaysL3 <- tabL2[,2][!is.na(tabL2[,2])] # days with L3 emergence

tabL3 = NULL
for( ind in 1:length(DaysL3)) {
  i <- DaysL3[ind] # day of development
  DevRateAccumul <- DevRate0
  j <- i
  
  while((DevRateAccumul < 1) & (j <= ndays)) {
      expr <- (TempVF$seq == j)
      Templ <- TempVF$Temp[expr]
      DevRateTaylor <- (Rm_L3*exp((-1/2)*(((Templ - Tm_L3)/To_L3)^2)))
      j <- j + 1
      DevRateAccumul <- DevRateAccumul + DevRateTaylor
    }

  if(DevRateAccumul >= 1) {
    tabL3 <- rbind(tabL3, cbind(i, j, DevRateAccumul))
    colnames(tabL3) <- c("first day", "last day", "DevRateAccumul")
  }
  
  if(DevRateAccumul < 1) {
    tabL3 <- rbind(tabL3, cbind(i, NA, DevRateAccumul))
    colnames(tabL3) <- c("first day", "last day", "DevRateAccumul")
  }
  
}

# to add the number of individuals per emergence date
popL3 <- tabL2$popL2[!is.na(tabL2[,2])]
tabL3 <- data.frame(tabL3, popL3)

tabL3VF <- tabL3[rep(row.names(tabL3), tabL3$popL3), 1:4]


resL3 <- cbind(min(tabL3VF[,1],na.rm=T), max(tabL3VF[,1],na.rm=T), 
                            min(tabL3VF[,2],na.rm=T), max(tabL3VF[,2],na.rm=T), 
                            mean(tabL3VF[,2]-tabL3VF[,1],na.rm=T))
colnames(resL3) <-  c("min First day", "max First day", "min Last day",
                      "max Last day", "mean duration")


##


#### L4 development ####

DaysL4 <- tabL3[,2][!is.na(tabL3[,2])] # days with L4 emergence

tabL4 = NULL
for( ind in 1:length(DaysL4)) {
  i <- DaysL4[ind] # day of development
  DevRateAccumul <- DevRate0
  j <- i
  
  while((DevRateAccumul < 1) & (j <= ndays)) {
      expr <- (TempVF$seq == j) 
      Templ <- TempVF$Temp[expr]
      DevRateTaylor <- (Rm_L4*exp((-1/2)*(((Templ - Tm_L4)/To_L4)^2)))
      j <- j + 1
      DevRateAccumul <- DevRateAccumul + DevRateTaylor
    }

  if(DevRateAccumul >= 1) {
    tabL4 <- rbind(tabL4, cbind(i, j, DevRateAccumul))
    colnames(tabL4) <- c("first day", "last day", "DevRateAccumul")
  }
  
  if(DevRateAccumul < 1) {
    tabL4 <- rbind(tabL4, cbind(i, NA, DevRateAccumul))
    colnames(tabL4) <- c("first day", "last day", "DevRateAccumul")
  }
  
}

# to add the number of individuals per emergence date
popL4 <- tabL3$popL3[!is.na(tabL3[,2])]
tabL4 <- data.frame(tabL4, popL4) 
tabL4VF <- tabL4[rep(row.names(tabL4), tabL4$popL4), 1:4]


resL4 <- cbind(min(tabL4VF[,1],na.rm=T), max(tabL4VF[,1],na.rm=T), 
                            min(tabL4VF[,2],na.rm=T), max(tabL4VF[,2],na.rm=T), 
                            mean(tabL4VF[,2]-tabL4VF[,1],na.rm=T))
colnames(resL4) <-  c("min First day", "max First day", "min Last day",
                      "max Last day", "mean duration")


##


#### L5 development ####

DaysL5 <- tabL4[,2][!is.na(tabL4[,2])] # days with L4 emergence

tabL5 = NULL
for( ind in 1:length(DaysL5)) {
  i <- DaysL5[ind] # day of development
  DevRateAccumul <- DevRate0
  j <- i
  
  while((DevRateAccumul < 1) & (j <= ndays)) {
      expr <- (TempVF$seq == j)
      Templ <- TempVF$Temp[expr]
      DevRateTaylor <- (Rm_L5*exp((-1/2)*(((Templ - Tm_L5)/To_L5)^2)))
      j <- j + 1
      DevRateAccumul <- DevRateAccumul + DevRateTaylor
    }

  if(DevRateAccumul >= 1) {
    tabL5 <- rbind(tabL5, cbind(i, j, DevRateAccumul))
    colnames(tabL5) <- c("first day", "last day", "DevRateAccumul")
  }
  
  if(DevRateAccumul < 1) {
    tabL5 <- rbind(tabL5, cbind(i, NA, DevRateAccumul))
    colnames(tabL5) <- c("first day", "last day", "DevRateAccumul")
  }
  
}

# to add the number of individuals per emergence date
popL5 <- tabL4[,4][!is.na(tabL4[,2])]
tabL5 <- data.frame(tabL5, popL5) 
tabL5VF <- tabL5[rep(row.names(tabL5), tabL5$popL5), 1:4]


resL5 <- cbind(min(tabL5VF[,1],na.rm=T), max(tabL5VF[,1],na.rm=T), 
                            min(tabL5VF[,2],na.rm=T), max(tabL5VF[,2],na.rm=T), 
                            mean(tabL5VF[,2]-tabL5VF[,1],na.rm=T))
colnames(resL5) <-  c("min First day", "max First day", "min Last day",
                      "max Last day", "mean duration")


##



#### Individuals percentage per instar ####

jmin = min(tabEgg[,1], na.rm=T) # first day of egg development
jmax = max(tabL5[,2], na.rm=T) # last day of larval development
tabnb = matrix(rep(0, (jmax - jmin + 1)*7),ncol=7) 
# day j, number of individual at egg stage, at L1 stage, ... at L5 stage => 7 variables
for (j in jmin:jmax) # loop on every day of development time from egg to l5
{
  tabnb[j - jmin + 1, 1] = j 
  
  # Eggs:
  expr = (j >= tabEgg[,1]) & (j <= tabEgg[,2]) # every day with egg 
  tabnb[j - jmin + 1, 2] = sum(tabEgg[expr,4]) # cumulative number of egg 
  # L1:
  expr = (j >= tabL1[,1]) & (j <= tabL1[,2])
  tabnb[j - jmin + 1, 3] = sum(tabL1[expr,4])
  # L2:
  expr = (j >= tabL2[,1]) & (j <= tabL2[,2])
  tabnb[j - jmin + 1, 4] = sum(tabL2[expr,4])
  # L3:
  expr = (j >= tabL3[,1]) & (j <= tabL3[,2])
  tabnb[j - jmin + 1, 5] = sum(tabL3[expr,4])
  # L4:
  expr = (j >= tabL4[,1]) & (j <= tabL4[,2])
  tabnb[j - jmin + 1, 6] = sum(tabL4[expr,4])
  # L5:
  expr = (j >= tabL5[,1]) & (j <= tabL5[,2])
  tabnb[j - jmin + 1, 7] = sum(tabL5[expr,4])
}

return(list(resegg = resegg, resL1 = resL1, resL2 = resL2, resL3 = resL3, 
            resL4 = resL4, resL5 = resL5, tabnb = tabnb))


} # function end


##########################################################################
# To call the function "param"
##########################################################################
 
x <- param(       Rm_egg = 0.02878, Tm_egg = 30.98523, To_egg = 32.31451,
                  Rm_L1 = 0.1064, Tm_L1 = 29.3348, To_L1 = 11.6322, 
                  Rm_L2 = 0.0689, Tm_L2 = 25.0753, To_L2 = 9.1466,
                  Rm_L4 = 0.05, Tm_L4 = 24, To_L4 = 10.9,
                  Rm_L5 = 0.05, Tm_L5 = 24, To_L5 = 10.9,
                  Rm_L3 = 0.04347826, Tm_L3 = 28, To_L3 = 23)
# or simply "x <- param()" if you would like to use the default values. 

x$resegg
x$resL1
x$resL2
x$resL3
x$resL4
x$resL5


# In x, you have a list with the results (x$resegg, x$resL1, x$resL2, x$resL3, x$resL4, x$resL5, x$tabnb) 
# For each life-stage, you have :
# - minimal and maximal first day of development
# - minimal and maximal last day of development
# - the mean duration for the development of this life stage
# - the number of individuals present at each date
# The days are given in Julian days (day 1 = 1st January of the year of adult flight and egg-laying).


# Table with predicted number of individuals per stage according to date
tabnb <- data.frame(x$tabnb) 
colnames(tabnb) <- c("Date", "Egg", "L1", "L2", "L3", "L4", "L5")
tabnb

